Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const path = require('node:path');
const core = require('@sentry/core');
const node = require('@sentry/node');
const debugBuild = require('../common/debug-build.js');

/**
 * Initializes the server-side of the Nuxt SDK
 *
 * @param options Configuration options for the SDK.
 */
function init(options) {
  const sentryOptions = {
    ...options,
    defaultIntegrations: getNuxtDefaultIntegrations(options),
  };

  core.applySdkMetadata(sentryOptions, 'nuxt', ['nuxt', 'node']);

  const client = node.init(sentryOptions);

  core.getGlobalScope().addEventProcessor(lowQualityTransactionsFilter(options));
  core.getGlobalScope().addEventProcessor(clientSourceMapErrorFilter(options));

  return client;
}

/**
 * Filter out transactions for resource requests which we don't want to send to Sentry
 * for quota reasons.
 *
 * Only exported for testing
 */
function lowQualityTransactionsFilter(options) {
  return Object.assign(
    (event => {
      if (event.type !== 'transaction' || !event.transaction) {
        return event;
      }
      // We don't want to send transaction for file requests, so everything ending with a *.someExtension should be filtered out
      // path.extname will return an empty string for normal page requests
      if (path.extname(event.transaction)) {
        options.debug &&
          debugBuild.DEBUG_BUILD &&
          core.logger.log('NuxtLowQualityTransactionsFilter filtered transaction: ', event.transaction);
        return null;
      }
      return event;
    }) ,
    { id: 'NuxtLowQualityTransactionsFilter' },
  );
}

/**
 * The browser devtools try to get the source maps, but as client source maps may not be available there is going to be an error (no problem for the application though).
 *
 * Only exported for testing
 */
function clientSourceMapErrorFilter(options) {
  return Object.assign(
    (event => {
      const errorMsg = event.exception?.values?.[0]?.value;
      if (errorMsg?.match(/^ENOENT: no such file or directory, open '.*\/_nuxt\/.*\.js\.map'/)) {
        options.debug && debugBuild.DEBUG_BUILD && core.logger.log('NuxtClientSourceMapErrorFilter filtered error: ', errorMsg);
        return null;
      }
      return event;
    }) ,
    { id: 'NuxtClientSourceMapErrorFilter' },
  );
}

function getNuxtDefaultIntegrations(options) {
  return [
    ...node.getDefaultIntegrations(options).filter(integration => integration.name !== 'Http'),
    // The httpIntegration is added as defaultIntegration, so users can still overwrite it
    node.httpIntegration({
      instrumentation: {
        responseHook: () => {
          // Makes it possible to end the tracing span before closing the Vercel lambda (https://vercel.com/docs/functions/functions-api-reference#waituntil)
          core.vercelWaitUntil(flushSafelyWithTimeout());
        },
      },
    }),
  ];
}

/**
 * Flushes pending Sentry events with a 2-second timeout and in a way that cannot create unhandled promise rejections.
 */
async function flushSafelyWithTimeout() {
  try {
    debugBuild.DEBUG_BUILD && core.logger.log('Flushing events...');
    await core.flush(2000);
    debugBuild.DEBUG_BUILD && core.logger.log('Done flushing events');
  } catch (e) {
    debugBuild.DEBUG_BUILD && core.logger.log('Error while flushing events:\n', e);
  }
}

exports.clientSourceMapErrorFilter = clientSourceMapErrorFilter;
exports.flushSafelyWithTimeout = flushSafelyWithTimeout;
exports.init = init;
exports.lowQualityTransactionsFilter = lowQualityTransactionsFilter;
//# sourceMappingURL=sdk.js.map
