import { createResolver, defineNuxtModule, addPluginTemplate, addPlugin, addServerPlugin, addTemplate } from '@nuxt/kit';
import { consoleSandbox, logger } from '@sentry/core';
import * as path from 'path';
import { existsSync } from 'node:fs';
import * as fs from 'fs';
import { sentryRollupPlugin } from '@sentry/rollup-plugin';
import { sentryVitePlugin } from '@sentry/vite-plugin';

function findDefaultSdkInitFile(type, nuxt) {
  const possibleFileExtensions = ["ts", "js", "mjs", "cjs", "mts", "cts"];
  const relativePaths = [];
  if (type === "server") {
    for (const ext of possibleFileExtensions) {
      relativePaths.push(`sentry.${type}.config.${ext}`);
      relativePaths.push(path.join("public", `instrument.${type}.${ext}`));
    }
  } else {
    for (const ext of possibleFileExtensions) {
      relativePaths.push(`sentry.${type}.config.${ext}`);
    }
  }
  const layers = [...nuxt?.options._layers ?? []].reverse();
  for (const layer of layers) {
    for (const relativePath of relativePaths) {
      const fullPath = path.resolve(layer.cwd, relativePath);
      if (fs.existsSync(fullPath)) {
        return fullPath;
      }
    }
  }
  const cwd = process.cwd();
  for (const relativePath of relativePaths) {
    const fullPath = path.resolve(cwd, relativePath);
    if (fs.existsSync(fullPath)) {
      return fullPath;
    }
  }
  return void 0;
}
function getFilenameFromNodeStartCommand(nodeCommand) {
  const regex = /[^/\\]+\.[^/\\]+$/;
  const match = nodeCommand.match(regex);
  return match ? match[0] : null;
}
const SENTRY_WRAPPED_ENTRY = "?sentry-query-wrapped-entry";
const SENTRY_WRAPPED_FUNCTIONS = "?sentry-query-wrapped-functions=";
const SENTRY_REEXPORTED_FUNCTIONS = "?sentry-query-reexported-functions=";
const QUERY_END_INDICATOR = "SENTRY-QUERY-END";
function removeSentryQueryFromPath(url) {
  const regex = new RegExp(`\\${SENTRY_WRAPPED_ENTRY}.*?\\${QUERY_END_INDICATOR}`);
  return url.replace(regex, "");
}
function extractFunctionReexportQueryParameters(query) {
  const wrapRegex = new RegExp(
    `\\${SENTRY_WRAPPED_FUNCTIONS}(.*?)(\\${QUERY_END_INDICATOR}|\\${SENTRY_REEXPORTED_FUNCTIONS})`
  );
  const reexportRegex = new RegExp(`\\${SENTRY_REEXPORTED_FUNCTIONS}(.*?)(\\${QUERY_END_INDICATOR})`);
  const wrapMatch = query.match(wrapRegex);
  const reexportMatch = query.match(reexportRegex);
  const wrap = wrapMatch?.[1]?.split(",").filter((param) => param !== "").map((str) => str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&")) || [];
  const reexport = reexportMatch?.[1]?.split(",").filter((param) => param !== "" && param !== "default").map((str) => str.replace(/[.*+?^${}()|[\]\\]/g, "\\$&")) || [];
  return { wrap, reexport };
}
function constructWrappedFunctionExportQuery(exportedBindings, entrypointWrappedFunctions, debug) {
  const functionsToExport = {
    wrap: [],
    reexport: []
  };
  Object.values(exportedBindings || {}).forEach(
    (functions) => functions.forEach((fn) => {
      if (entrypointWrappedFunctions.includes(fn)) {
        functionsToExport.wrap.push(fn);
      } else {
        functionsToExport.reexport.push(fn);
      }
    })
  );
  if (debug && functionsToExport.wrap.length === 0) {
    consoleSandbox(
      () => (
        // eslint-disable-next-line no-console
        console.warn(
          "[Sentry] No functions found to wrap. In case the server needs to export async functions other than `handler` or  `server`, consider adding the name(s) to Sentry's build options `sentry.experimental_entrypointWrappedFunctions` in `nuxt.config.ts`."
        )
      )
    );
  }
  const wrapQuery = functionsToExport.wrap.length ? `${SENTRY_WRAPPED_FUNCTIONS}${functionsToExport.wrap.join(",")}` : "";
  const reexportQuery = functionsToExport.reexport.length ? `${SENTRY_REEXPORTED_FUNCTIONS}${functionsToExport.reexport.join(",")}` : "";
  return [wrapQuery, reexportQuery].join("");
}
function constructFunctionReExport(pathWithQuery, entryId) {
  const { wrap: wrapFunctions, reexport: reexportFunctions } = extractFunctionReexportQueryParameters(pathWithQuery);
  return wrapFunctions.reduce(
    (functionsCode, currFunctionName) => functionsCode.concat(
      `async function ${currFunctionName}_sentryWrapped(...args) {
  const res = await import(${JSON.stringify(entryId)});
  return res.${currFunctionName}.call(this, ...args);
}
export { ${currFunctionName}_sentryWrapped as ${currFunctionName} };
`
    ),
    ""
  ).concat(
    reexportFunctions.reduce(
      (functionsCode, currFunctionName) => functionsCode.concat(`export { ${currFunctionName} } from ${JSON.stringify(entryId)};`),
      ""
    )
  );
}
function addOTelCommonJSImportAlias(nuxt) {
  if (!nuxt.options.dev) {
    return;
  }
  if (!nuxt.options.alias) {
    nuxt.options.alias = {};
  }
  if (!nuxt.options.alias["@opentelemetry/resources"]) {
    nuxt.options.alias["@opentelemetry/resources"] = "@opentelemetry/resources/build/src/index.js";
  }
}

const SERVER_CONFIG_FILENAME = "sentry.server.config";
function addServerConfigToBuild(moduleOptions, nitro, serverConfigFile) {
  nitro.hooks.hook("rollup:before", (nitro2, rollupConfig) => {
    if (rollupConfig?.plugins === null || rollupConfig?.plugins === void 0) {
      rollupConfig.plugins = [];
    } else if (!Array.isArray(rollupConfig.plugins)) {
      rollupConfig.plugins = [rollupConfig.plugins];
    }
    rollupConfig.plugins.push(injectServerConfigPlugin(nitro2, serverConfigFile, moduleOptions.debug));
  });
}
function addSentryTopImport(moduleOptions, nitro) {
  nitro.hooks.hook("close", async () => {
    const fileNameFromCommand = nitro.options.commands.preview && getFilenameFromNodeStartCommand(nitro.options.commands.preview);
    const presetsWithServerFile = ["netlify"];
    const entryFileName = fileNameFromCommand ? fileNameFromCommand : typeof nitro.options.rollupConfig?.output.entryFileNames === "string" ? nitro.options.rollupConfig?.output.entryFileNames : presetsWithServerFile.includes(nitro.options.preset) ? "server.mjs" : "index.mjs";
    const serverDirResolver = createResolver(nitro.options.output.serverDir);
    const entryFilePath = serverDirResolver.resolve(entryFileName);
    try {
      fs.readFile(entryFilePath, "utf8", (err, data) => {
        const updatedContent = `import './${SERVER_CONFIG_FILENAME}.mjs';
${data}`;
        fs.writeFile(entryFilePath, updatedContent, "utf8", () => {
          if (moduleOptions.debug) {
            console.log(
              `[Sentry] Successfully added the Sentry import to the server entry file "\`${entryFilePath}\`"`
            );
          }
        });
      });
    } catch (err) {
      if (moduleOptions.debug) {
        console.warn(
          `[Sentry] An error occurred when trying to add the Sentry import to the server entry file "\`${entryFilePath}\`":`,
          err
        );
      }
    }
  });
}
function addDynamicImportEntryFileWrapper(nitro, serverConfigFile, moduleOptions) {
  if (!nitro.options.rollupConfig) {
    nitro.options.rollupConfig = { output: {} };
  }
  if (nitro.options.rollupConfig?.plugins === null || nitro.options.rollupConfig?.plugins === void 0) {
    nitro.options.rollupConfig.plugins = [];
  } else if (!Array.isArray(nitro.options.rollupConfig.plugins)) {
    nitro.options.rollupConfig.plugins = [nitro.options.rollupConfig.plugins];
  }
  nitro.options.rollupConfig.plugins.push(
    wrapEntryWithDynamicImport({
      resolvedSentryConfigPath: createResolver(nitro.options.srcDir).resolve(`/${serverConfigFile}`),
      experimental_entrypointWrappedFunctions: moduleOptions.experimental_entrypointWrappedFunctions
    })
  );
}
function injectServerConfigPlugin(nitro, serverConfigFile, debug) {
  const filePrefix = "\0virtual:sentry-server-config:";
  return {
    name: "rollup-plugin-inject-sentry-server-config",
    buildStart() {
      const configPath = createResolver(nitro.options.srcDir).resolve(`/${serverConfigFile}`);
      if (!existsSync(configPath)) {
        if (debug) {
          logger.log(`[Sentry] Sentry server config file not found: ${configPath}`);
        }
        return;
      }
      this.emitFile({
        type: "chunk",
        id: `${filePrefix}${serverConfigFile}`,
        fileName: `${SERVER_CONFIG_FILENAME}.mjs`
      });
    },
    resolveId(source) {
      if (source.startsWith(filePrefix)) {
        const originalFilePath = source.replace(filePrefix, "");
        const configPath = createResolver(nitro.options.rootDir).resolve(`/${originalFilePath}`);
        return { id: configPath };
      }
      return null;
    }
  };
}
function wrapEntryWithDynamicImport({
  resolvedSentryConfigPath,
  experimental_entrypointWrappedFunctions,
  debug
}) {
  const resolutionIdPrefix = "\0raw";
  return {
    name: "sentry-wrap-entry-with-dynamic-import",
    async resolveId(source, importer, options) {
      if (source.includes(`/${SERVER_CONFIG_FILENAME}`)) {
        return { id: source, moduleSideEffects: true };
      }
      if (source === "import-in-the-middle/hook.mjs") {
        return { id: source, moduleSideEffects: true, external: true };
      }
      if (options.isEntry && source.includes(".mjs") && !source.includes(`.mjs${SENTRY_WRAPPED_ENTRY}`)) {
        const resolution = await this.resolve(source, importer, options);
        if (!resolution || resolution?.external)
          return resolution;
        const moduleInfo = await this.load(resolution);
        moduleInfo.moduleSideEffects = true;
        return resolution.id.includes(`.mjs${SENTRY_WRAPPED_ENTRY}`) ? resolution.id : `${resolutionIdPrefix}${resolution.id.concat(SENTRY_WRAPPED_ENTRY).concat(
          constructWrappedFunctionExportQuery(
            moduleInfo.exportedBindings,
            experimental_entrypointWrappedFunctions,
            debug
          )
        ).concat(QUERY_END_INDICATOR)}`;
      }
      return null;
    },
    load(id) {
      if (id.includes(`.mjs${SENTRY_WRAPPED_ENTRY}`)) {
        const entryId = removeSentryQueryFromPath(id).slice(resolutionIdPrefix.length);
        const reExportedFunctions = id.includes(SENTRY_WRAPPED_FUNCTIONS) || id.includes(SENTRY_REEXPORTED_FUNCTIONS) ? constructFunctionReExport(id, entryId) : "";
        return (
          // Regular `import` of the Sentry config
          `import ${JSON.stringify(resolvedSentryConfigPath)};
import(${JSON.stringify(entryId)});
import 'import-in-the-middle/hook.mjs';
${reExportedFunctions}
`
        );
      }
      return null;
    }
  };
}

function setupSourceMaps(moduleOptions, nuxt) {
  const isDebug = moduleOptions.debug;
  const sourceMapsUploadOptions = moduleOptions.sourceMapsUploadOptions || {};
  const sourceMapsEnabled = sourceMapsUploadOptions.enabled ?? true;
  let shouldDeleteFilesFallback = { client: true, server: true };
  nuxt.hook("modules:done", () => {
    if (sourceMapsEnabled && !nuxt.options.dev) {
      const previousSourceMapSettings = changeNuxtSourceMapSettings(nuxt, moduleOptions);
      shouldDeleteFilesFallback = {
        client: previousSourceMapSettings.client === "unset",
        server: previousSourceMapSettings.server === "unset"
      };
      if (isDebug && !sourceMapsUploadOptions.sourcemaps?.filesToDeleteAfterUpload && (shouldDeleteFilesFallback.client || shouldDeleteFilesFallback.server)) {
        consoleSandbox(
          () => (
            // eslint-disable-next-line no-console
            console.log(
              "[Sentry] As Sentry enabled `'hidden'` source maps, source maps will be automatically deleted after uploading them to Sentry."
            )
          )
        );
      }
    }
  });
  nuxt.hook("vite:extendConfig", async (viteConfig, env) => {
    if (sourceMapsEnabled && viteConfig.mode !== "development") {
      const runtime = env.isServer ? "server" : env.isClient ? "client" : void 0;
      const nuxtSourceMapSetting = extractNuxtSourceMapSetting(nuxt, runtime);
      viteConfig.build = viteConfig.build || {};
      const viteSourceMap = viteConfig.build.sourcemap;
      validateDifferentSourceMapSettings({
        nuxtSettingKey: `sourcemap.${runtime}`,
        nuxtSettingValue: nuxtSourceMapSetting,
        otherSettingKey: "viteConfig.build.sourcemap",
        otherSettingValue: viteSourceMap
      });
      if (isDebug) {
        consoleSandbox(() => {
          if (!runtime) {
            console.log("[Sentry] Cannot detect runtime (client/server) inside hook 'vite:extendConfig'.");
          } else {
            console.log(`[Sentry] Adding Sentry Vite plugin to the ${runtime} runtime.`);
          }
        });
      }
      viteConfig.plugins = viteConfig.plugins || [];
      viteConfig.plugins.push(sentryVitePlugin(getPluginOptions(moduleOptions, shouldDeleteFilesFallback)));
    }
  });
  nuxt.hook("nitro:config", (nitroConfig) => {
    if (sourceMapsEnabled && !nitroConfig.dev) {
      if (!nitroConfig.rollupConfig) {
        nitroConfig.rollupConfig = {};
      }
      if (nitroConfig.rollupConfig.plugins === null || nitroConfig.rollupConfig.plugins === void 0) {
        nitroConfig.rollupConfig.plugins = [];
      } else if (!Array.isArray(nitroConfig.rollupConfig.plugins)) {
        nitroConfig.rollupConfig.plugins = [nitroConfig.rollupConfig.plugins];
      }
      validateNitroSourceMapSettings(nuxt, nitroConfig, moduleOptions);
      if (isDebug) {
        consoleSandbox(() => {
          console.log("[Sentry] Adding Sentry Rollup plugin to the server runtime.");
        });
      }
      nitroConfig.rollupConfig.plugins.push(
        sentryRollupPlugin(getPluginOptions(moduleOptions, shouldDeleteFilesFallback))
      );
    }
  });
}
function normalizePath(path) {
  return path.replace(/^(\.\.\/)+/, "./");
}
function getPluginOptions(moduleOptions, shouldDeleteFilesFallback) {
  const sourceMapsUploadOptions = moduleOptions.sourceMapsUploadOptions || {};
  const shouldDeleteFilesAfterUpload = shouldDeleteFilesFallback?.client || shouldDeleteFilesFallback?.server;
  const fallbackFilesToDelete = [
    ...shouldDeleteFilesFallback?.client ? [".*/**/public/**/*.map"] : [],
    ...shouldDeleteFilesFallback?.server ? [".*/**/server/**/*.map", ".*/**/output/**/*.map", ".*/**/function/**/*.map"] : []
  ];
  if (typeof sourceMapsUploadOptions.sourcemaps?.filesToDeleteAfterUpload === "undefined" && shouldDeleteFilesAfterUpload) {
    consoleSandbox(() => {
      console.log(
        `[Sentry] Setting \`sentry.sourceMapsUploadOptions.sourcemaps.filesToDeleteAfterUpload: [${fallbackFilesToDelete.map((path) => `"${path}"`).join(", ")}]\` to delete generated source maps after they were uploaded to Sentry.`
      );
    });
  }
  return {
    org: sourceMapsUploadOptions.org ?? process.env.SENTRY_ORG,
    project: sourceMapsUploadOptions.project ?? process.env.SENTRY_PROJECT,
    authToken: sourceMapsUploadOptions.authToken ?? process.env.SENTRY_AUTH_TOKEN,
    telemetry: sourceMapsUploadOptions.telemetry ?? true,
    url: sourceMapsUploadOptions.url ?? process.env.SENTRY_URL,
    debug: moduleOptions.debug ?? false,
    silent: sourceMapsUploadOptions.silent ?? false,
    errorHandler: sourceMapsUploadOptions.errorHandler,
    release: {
      name: sourceMapsUploadOptions.release?.name,
      ...moduleOptions?.unstable_sentryBundlerPluginOptions?.release
    },
    _metaOptions: {
      telemetry: {
        metaFramework: "nuxt"
      }
    },
    ...moduleOptions?.unstable_sentryBundlerPluginOptions,
    sourcemaps: {
      // The server/client files are in different places depending on the nitro preset (e.g. '.output/server' or '.netlify/functions-internal/server')
      // We cannot determine automatically how the build folder looks like (depends on the preset), so we have to accept that source maps are uploaded multiple times (with the vitePlugin for Nuxt and the rollupPlugin for Nitro).
      // If we could know where the server/client assets are located, we could do something like this (based on the Nitro preset): isNitro ? ['./.output/server/**/*'] : ['./.output/public/**/*'],
      assets: sourceMapsUploadOptions.sourcemaps?.assets ?? void 0,
      ignore: sourceMapsUploadOptions.sourcemaps?.ignore ?? void 0,
      filesToDeleteAfterUpload: sourceMapsUploadOptions.sourcemaps?.filesToDeleteAfterUpload ? sourceMapsUploadOptions.sourcemaps?.filesToDeleteAfterUpload : shouldDeleteFilesFallback?.server || shouldDeleteFilesFallback?.client ? fallbackFilesToDelete : void 0,
      rewriteSources: (source) => normalizePath(source),
      ...moduleOptions?.unstable_sentryBundlerPluginOptions?.sourcemaps
    }
  };
}
function extractNuxtSourceMapSetting(nuxt, runtime) {
  if (!runtime) {
    return void 0;
  } else {
    return typeof nuxt.options?.sourcemap === "boolean" || typeof nuxt.options?.sourcemap === "string" ? nuxt.options.sourcemap : nuxt.options?.sourcemap?.[runtime];
  }
}
function changeNuxtSourceMapSettings(nuxt, sentryModuleOptions) {
  nuxt.options.sourcemap = nuxt.options.sourcemap ?? { server: void 0, client: void 0 };
  let previousUserSourceMapSetting = {
    client: void 0,
    server: void 0
  };
  const nuxtSourceMap = nuxt.options.sourcemap;
  if (typeof nuxtSourceMap === "string" || typeof nuxtSourceMap === "boolean" || typeof nuxtSourceMap === "undefined") {
    switch (nuxtSourceMap) {
      case false:
        warnExplicitlyDisabledSourceMap("sourcemap");
        previousUserSourceMapSetting = { client: "disabled", server: "disabled" };
        break;
      case "hidden":
      case true:
        logKeepEnabledSourceMapSetting(sentryModuleOptions, "sourcemap", nuxtSourceMap.toString());
        previousUserSourceMapSetting = { client: "enabled", server: "enabled" };
        break;
      case void 0:
        nuxt.options.sourcemap = { server: "hidden", client: "hidden" };
        logSentryEnablesSourceMap("sourcemap.client", "hidden");
        logSentryEnablesSourceMap("sourcemap.server", "hidden");
        previousUserSourceMapSetting = { client: "unset", server: "unset" };
        break;
    }
  } else {
    if (nuxtSourceMap.client === false) {
      warnExplicitlyDisabledSourceMap("sourcemap.client");
      previousUserSourceMapSetting.client = "disabled";
    } else if (["hidden", true].includes(nuxtSourceMap.client)) {
      logKeepEnabledSourceMapSetting(sentryModuleOptions, "sourcemap.client", nuxtSourceMap.client.toString());
      previousUserSourceMapSetting.client = "enabled";
    } else {
      nuxt.options.sourcemap.client = "hidden";
      logSentryEnablesSourceMap("sourcemap.client", "hidden");
      previousUserSourceMapSetting.client = "unset";
    }
    if (nuxtSourceMap.server === false) {
      warnExplicitlyDisabledSourceMap("sourcemap.server");
      previousUserSourceMapSetting.server = "disabled";
    } else if (["hidden", true].includes(nuxtSourceMap.server)) {
      logKeepEnabledSourceMapSetting(sentryModuleOptions, "sourcemap.server", nuxtSourceMap.server.toString());
      previousUserSourceMapSetting.server = "enabled";
    } else {
      nuxt.options.sourcemap.server = "hidden";
      logSentryEnablesSourceMap("sourcemap.server", "hidden");
      previousUserSourceMapSetting.server = "unset";
    }
  }
  return previousUserSourceMapSetting;
}
function validateNitroSourceMapSettings(nuxt, nitroConfig, sentryModuleOptions) {
  const isDebug = sentryModuleOptions.debug;
  const nuxtSourceMap = extractNuxtSourceMapSetting(nuxt, "server");
  validateDifferentSourceMapSettings({
    nuxtSettingKey: "sourcemap.server",
    nuxtSettingValue: nuxtSourceMap,
    otherSettingKey: "nitro.sourceMap",
    otherSettingValue: nitroConfig.sourceMap
  });
  nitroConfig.rollupConfig = nitroConfig.rollupConfig || {};
  nitroConfig.rollupConfig.output = nitroConfig.rollupConfig.output || { sourcemap: void 0 };
  const nitroRollupSourceMap = nitroConfig.rollupConfig.output.sourcemap;
  if (typeof nitroRollupSourceMap !== "undefined" && ["hidden", "inline", true, false].includes(nitroRollupSourceMap)) {
    const settingKey = "nitro.rollupConfig.output.sourcemap";
    validateDifferentSourceMapSettings({
      nuxtSettingKey: "sourcemap.server",
      nuxtSettingValue: nuxtSourceMap,
      otherSettingKey: settingKey,
      otherSettingValue: nitroRollupSourceMap
    });
  }
  nitroConfig.rollupConfig.output.sourcemapExcludeSources = false;
  if (isDebug) {
    consoleSandbox(() => {
      console.log(
        "[Sentry] Set `sourcemapExcludeSources: false` in the Nuxt config (`nitro.rollupConfig.output`). Source maps will now include the actual code to be able to un-minify code snippets in Sentry."
      );
    });
  }
}
function validateDifferentSourceMapSettings({
  nuxtSettingKey,
  nuxtSettingValue,
  otherSettingKey,
  otherSettingValue
}) {
  if (nuxtSettingValue !== otherSettingValue) {
    consoleSandbox(() => {
      console.warn(
        `[Sentry] Source map generation settings are conflicting. Sentry uses \`${nuxtSettingKey}: ${nuxtSettingValue}\`. However, a conflicting setting was discovered (\`${otherSettingKey}: ${otherSettingValue}\`). This setting was probably explicitly set in your configuration. Sentry won't override this setting but it may affect source maps generation and upload. Without source maps, code snippets on the Sentry Issues page will remain minified.`
      );
    });
  }
}
function logKeepEnabledSourceMapSetting(sentryNuxtModuleOptions, settingKey, settingValue) {
  if (sentryNuxtModuleOptions.debug) {
    consoleSandbox(() => {
      console.log(
        `[Sentry] \`${settingKey}\` is enabled with \`${settingValue}\`. This will correctly un-minify the code snippet on the Sentry Issue Details page.`
      );
    });
  }
}
function warnExplicitlyDisabledSourceMap(settingKey) {
  consoleSandbox(() => {
    console.warn(
      `[Sentry] We discovered \`${settingKey}\` is set to \`false\`. This setting is either a default setting or was explicitly set in your configuration. Sentry won't override this setting. Without source maps, code snippets on the Sentry Issues page will remain minified. To show unminified code, enable source maps in \`${settingKey}\` (e.g. by setting them to \`'hidden'\`).`
    );
  });
}
function logSentryEnablesSourceMap(settingKey, settingValue) {
  consoleSandbox(() => {
    console.log(`[Sentry] Enabled source map generation in the build options with \`${settingKey}: ${settingValue}\`.`);
  });
}

const module = defineNuxtModule({
  meta: {
    name: "@sentry/nuxt/module",
    configKey: "sentry",
    compatibility: {
      nuxt: ">=3.7.0"
    }
  },
  defaults: {},
  setup(moduleOptionsParam, nuxt) {
    if (moduleOptionsParam?.enabled === false) {
      return;
    }
    const moduleOptions = {
      ...moduleOptionsParam,
      autoInjectServerSentry: moduleOptionsParam.autoInjectServerSentry,
      experimental_entrypointWrappedFunctions: moduleOptionsParam.experimental_entrypointWrappedFunctions || [
        "default",
        "handler",
        "server"
      ]
    };
    const moduleDirResolver = createResolver(import.meta.url);
    const buildDirResolver = createResolver(nuxt.options.buildDir);
    const clientConfigFile = findDefaultSdkInitFile("client", nuxt);
    if (clientConfigFile) {
      addPluginTemplate({
        mode: "client",
        filename: "sentry-client-config.mjs",
        order: 0,
        // Dynamic import of config file to wrap it within a Nuxt context (here: defineNuxtPlugin)
        // Makes it possible to call useRuntimeConfig() in the user-defined sentry config file
        getContents: () => `
          import { defineNuxtPlugin } from "#imports";

          export default defineNuxtPlugin({
            name: 'sentry-client-config',
            async setup() {
              await import("${buildDirResolver.resolve(`/${clientConfigFile}`)}")
            }
          });`
      });
      addPlugin({
        src: moduleDirResolver.resolve("./runtime/plugins/sentry.client"),
        mode: "client",
        order: 1
      });
    }
    const serverConfigFile = findDefaultSdkInitFile("server", nuxt);
    if (serverConfigFile) {
      addServerPlugin(moduleDirResolver.resolve("./runtime/plugins/sentry.server"));
      addPlugin({
        src: moduleDirResolver.resolve("./runtime/plugins/route-detector.server"),
        mode: "server"
      });
    }
    if (clientConfigFile || serverConfigFile) {
      setupSourceMaps(moduleOptions, nuxt);
    }
    addOTelCommonJSImportAlias(nuxt);
    const pagesDataTemplate = addTemplate({
      filename: "sentry--nuxt-pages-data.mjs",
      // Initial empty array (later filled in pages:extend hook)
      // Template needs to be created in the root-level of the module to work
      getContents: () => "export default [];"
    });
    nuxt.hooks.hook("pages:extend", (pages) => {
      pagesDataTemplate.getContents = () => {
        const pagesSubset = pages.map((page) => ({ file: page.file, path: page.path })).filter((page) => {
          return page.path.includes(":") || page?.file?.includes("[");
        });
        return `export default ${JSON.stringify(pagesSubset, null, 2)};`;
      };
    });
    nuxt.hooks.hook("nitro:init", (nitro) => {
      if (serverConfigFile?.includes(".server.config")) {
        if (nitro.options.dev) {
          consoleSandbox(() => {
            console.log(
              "[Sentry] Your application is running in development mode. Note: @sentry/nuxt does not work as expected on the server-side (Nitro). Errors are reported, but tracing does not work."
            );
          });
        }
        consoleSandbox(() => {
          const serverDir = nitro.options.output.serverDir;
          if (serverDir.includes(".netlify") || !!process.env.NETLIFY) {
            console.warn(
              "[Sentry] Warning: The Sentry SDK detected a Netlify build. Server-side support for the Sentry Nuxt SDK on Netlify is currently unreliable due to technical limitations of serverless functions. Traces are not collected, and errors may occasionally not be reported. For more information on setting up Sentry on the Nuxt server-side, please refer to the documentation: https://docs.sentry.io/platforms/javascript/guides/nuxt/install/"
            );
          }
          if (serverDir.includes(".vercel") || !!process.env.VERCEL) {
            console.warn(
              "[Sentry] Warning: The Sentry SDK detected a Vercel build. The Sentry Nuxt SDK currently does not support tracing on Vercel. For more information on setting up Sentry on the Nuxt server-side, please refer to the documentation: https://docs.sentry.io/platforms/javascript/guides/nuxt/install/"
            );
          }
        });
        if (moduleOptions.autoInjectServerSentry !== "experimental_dynamic-import") {
          addServerConfigToBuild(moduleOptions, nitro, serverConfigFile);
          if (moduleOptions.debug) {
            const serverDirResolver = createResolver(nitro.options.output.serverDir);
            const serverConfigPath = serverDirResolver.resolve("sentry.server.config.mjs");
            const serverConfigRelativePath = `.${path.sep}${path.relative(nitro.options.rootDir, serverConfigPath)}`;
            consoleSandbox(() => {
              console.log(
                `[Sentry] Using your \`${serverConfigFile}\` file for the server-side Sentry configuration. Make sure to add the Node option \`import\` to the Node command where you deploy and/or run your application. This preloads the Sentry configuration at server startup. You can do this via a command-line flag (\`node --import ${serverConfigRelativePath} [...]\`) or via an environment variable (\`NODE_OPTIONS='--import ${serverConfigRelativePath}' node [...]\`).`
              );
            });
          }
        }
        if (moduleOptions.autoInjectServerSentry === "top-level-import") {
          addSentryTopImport(moduleOptions, nitro);
        }
        if (moduleOptions.autoInjectServerSentry === "experimental_dynamic-import") {
          addDynamicImportEntryFileWrapper(nitro, serverConfigFile, moduleOptions);
          if (moduleOptions.debug) {
            consoleSandbox(() => {
              console.log(
                "[Sentry] Wrapping the server entry file with a dynamic `import()`, so Sentry can be preloaded before the server initializes."
              );
            });
          }
        }
      }
    });
  }
});

export { module as default };
