import { setAsyncLocalStorageAsyncContextStrategy, wrapRequestHandler } from "@sentry/cloudflare";
import { getDefaultIsolationScope, getIsolationScope, getTraceData, logger } from "@sentry/core";
import { sentryCaptureErrorHook } from "../hooks/captureErrorHook.js";
import { updateRouteBeforeResponse } from "../hooks/updateRouteBeforeResponse.js";
import { addSentryTracingMetaTags } from "../utils.js";
function isEventType(event) {
  if (event === null || typeof event !== "object") return false;
  return (
    // basic properties
    "protocol" in event && "host" in event && typeof event.protocol === "string" && typeof event.host === "string" && // context property
    "context" in event && typeof event.context === "object" && event.context !== null && // context.cf properties
    "cf" in event.context && typeof event.context.cf === "object" && event.context.cf !== null && // context.cloudflare properties
    "cloudflare" in event.context && typeof event.context.cloudflare === "object" && event.context.cloudflare !== null && "context" in event.context.cloudflare
  );
}
export const sentryCloudflareNitroPlugin = (optionsOrFn) => (nitroApp) => {
  const traceDataMap = /* @__PURE__ */ new WeakMap();
  nitroApp.localFetch = new Proxy(nitroApp.localFetch, {
    async apply(handlerTarget, handlerThisArg, handlerArgs) {
      setAsyncLocalStorageAsyncContextStrategy();
      const cloudflareOptions = typeof optionsOrFn === "function" ? optionsOrFn(nitroApp) : optionsOrFn;
      const pathname = handlerArgs[0];
      const event = handlerArgs[1];
      if (!isEventType(event)) {
        logger.log("Nitro Cloudflare plugin did not detect a Cloudflare event type. Won't patch Cloudflare handler.");
        return handlerTarget.apply(handlerThisArg, handlerArgs);
      } else {
        const url = `${event.protocol}${event.protocol.endsWith(":") ? "" : ":"}//${event.host}${pathname}`;
        const request = new Request(url, {
          method: event.method,
          headers: event.headers,
          cf: event.context.cf
        });
        const requestHandlerOptions = {
          options: cloudflareOptions,
          request,
          context: event.context.cloudflare.context
        };
        return wrapRequestHandler(requestHandlerOptions, () => {
          const isolationScope = getIsolationScope();
          const newIsolationScope = isolationScope === getDefaultIsolationScope() ? isolationScope.clone() : isolationScope;
          const traceData = getTraceData();
          if (traceData && Object.keys(traceData).length > 0) {
            traceDataMap.set(event.context.cf, traceData);
            logger.log("Stored trace data for later use in HTML meta-tags: ", traceData);
          }
          logger.log(
            `Patched Cloudflare handler (\`nitroApp.localFetch\`). ${isolationScope === newIsolationScope ? "Using existing" : "Created new"} isolation scope.`
          );
          return handlerTarget.apply(handlerThisArg, handlerArgs);
        });
      }
    }
  });
  nitroApp.hooks.hook("beforeResponse", updateRouteBeforeResponse);
  nitroApp.hooks.hook("render:html", (html, { event }) => {
    const storedTraceData = event?.context?.cf ? traceDataMap.get(event.context.cf) : void 0;
    if (storedTraceData && Object.keys(storedTraceData).length > 0) {
      logger.log("Using stored trace data for HTML meta-tags: ", storedTraceData);
      addSentryTracingMetaTags(html.head, storedTraceData);
    } else {
      addSentryTracingMetaTags(html.head);
    }
  });
  nitroApp.hooks.hook("error", sentryCaptureErrorHook);
};
